package com.unissoft.mqtt;

import org.eclipse.paho.client.mqttv3.MqttConnectOptions;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.integration.annotation.MessagingGateway;
import org.springframework.integration.annotation.ServiceActivator;
import org.springframework.integration.channel.DirectChannel;
import org.springframework.integration.mqtt.core.DefaultMqttPahoClientFactory;
import org.springframework.integration.mqtt.core.MqttPahoClientFactory;
import org.springframework.integration.mqtt.outbound.MqttPahoMessageHandler;
import org.springframework.integration.mqtt.support.MqttHeaders;
import org.springframework.messaging.MessageChannel;
import org.springframework.messaging.MessageHandler;
import org.springframework.messaging.handler.annotation.Header;
import org.springframework.stereotype.Component;

/**
 * @author qingsong.han
 * @description: mqtt config class
 * @create 2022-10-25 14:40
 */
@Configuration
@SuppressWarnings("all")
public class MqttConfig {

    @Value("${mqtt.serverURIs}")
    private String mqttServerURIs;
    @Value("${mqtt.username}")
    private String mqttUserName;
    @Value("${mqtt.password}")
    private String mqttPwd;
    @Value("${mqtt.keepAliveInterval}")
    private Integer keepAliveInterval;
    @Value("${mqtt.cleanSession}")
    private Boolean cleanSession;

    /**
     * 创建设置MQTT Broker连接属性,如果使用SSL验证,也在这里设置(构造一个默认的mqtt客户端操作bean)
     *
     * @return MqttPahoClientFactory
     */
    @Bean
    public MqttPahoClientFactory factory() {
        DefaultMqttPahoClientFactory factory = new DefaultMqttPahoClientFactory();
        MqttConnectOptions options = new MqttConnectOptions();
        //如果配置了用户密码这里需要对应设置
        options.setUserName(mqttUserName);
        options.setPassword(mqttPwd.toCharArray());
        String[] array = mqttServerURIs.split(",");
        options.setServerURIs(array);
        // 设为false，客户端掉线后 服务器端不会清除session，当重连后可以接收之前订阅主题的消息。当客户端上线后会接受到它离线的这段时间的消息
        options.setCleanSession(cleanSession);
        // 设置会话心跳时间 单位为秒
        options.setKeepAliveInterval(keepAliveInterval);
        factory.setConnectionOptions(options);
        return factory;
    }

    /**
     * 出站-生产者
     */
    public static final String CHANNEL_NAME_OUT = "mqttOutputChannel";
    @Value("${mqtt.clientId}")
    private String clientId;
    @Value("${mqtt.qos}")
    private int qos;

    @Bean
    public MessageChannel mqttOutputChannel() {
        return new DirectChannel();
    }

    @Bean
    @ServiceActivator(inputChannel = MqttConfig.CHANNEL_NAME_OUT)
    public MessageHandler mqttOutbound(MqttPahoClientFactory factory) {
        MqttPahoMessageHandler messageHandler = new MqttPahoMessageHandler(clientId, factory);
        messageHandler.setAsync(true);
        messageHandler.setDefaultTopic("mqtt_default_topic"); // 默认主题不能为空
        return messageHandler;
    }

    @Component
    @MessagingGateway(defaultRequestChannel = MqttConfig.CHANNEL_NAME_OUT)
    interface MqttMessageSender {
        /**
         * 发送信息
         *
         * @param payload 发送的消息
         */
        void sendToMqtt(String payload);

        /**
         * 指定主题发送消息
         *
         * @param topic   主题
         * @param payload 消息主体
         */
        void sendToMqtt(@Header(MqttHeaders.TOPIC) String topic, String payload);

        /**
         * 指定主题、qos发送消息
         *
         * @param topic   主题
         * @param qos     对消息处理的几种机制。
         *                0 表示的是订阅者没收到消息不会再次发送，消息会丢失。
         *                1 表示的是会尝试重试，一直到接收到消息，但这种情况可能导致订阅者收到多次重复消息。
         *                2 多了一次去重的动作，确保订阅者收到的消息有一次。
         * @param payload 消息主体
         */
        void sendToMqtt(@Header(MqttHeaders.TOPIC) String topic, @Header(MqttHeaders.QOS) int qos, String payload);

        /**
         * 指定topic进行消息发送
         *
         * @param topic   自定义
         * @param qos     (0,1,2)
         * @param payload byte[]
         */
        void sendToMqtt(@Header(MqttHeaders.TOPIC) String topic, @Header(MqttHeaders.QOS) int qos, byte[] payload);
    }

}
